import sys
sys.path.append("../..")
from testcase_automaker.Utils.amazingutils import randoms
from allpairspy import AllPairs
import random
import copy


class http_params_generator(object):
    '''


    >>> params_generator = http_params_generator(parameters_structure={'name': {'type': 'string', 'value': '','range':['张三','李四'], 'iscompulsory': True},\
                                                                      'phone': {'type': 'number', 'value': '', 'iscompulsory': True},\
                                                                       'claimant': {'type': 'object', 'value': {'name': {'type': 'string', 'value': '', 'iscompulsory': True}\
                                                                        ,'phone': {'type': 'number', 'value': '', 'iscompulsory': True}}, 'iscompulsory': True},\
                                                                       'informations': {'type': 'array', 'value': [{'claimant': {'type': 'object', 'value': {'name': {'type': 'string', 'value': '', 'iscompulsory': True}\
                                                                        ,'phone': {'type': 'number', 'value': '', 'iscompulsory': True}}, 'iscompulsory': True}},\
                                                                       {'name': {'type': 'string', 'value': '', 'iscompulsory': True}}], 'iscompulsory': True}})

    >>> params_generator.get_params_num()
    7
    >>> params_generator.generate_params_list()
    >>> type(params_generator.generated_params_list)
    <class 'list'>
    >>> type(random.choice(params_generator.generated_params_list))
    <class 'dict'>
    >>> len(params_generator.generated_params_list)
    7

    >>> params_generator = http_params_generator(parameters_structure={})
    >>> params_generator.get_params_num()
    0
    >>> params_generator.generate_params_list()
    >>> type(params_generator.generated_params_list)
    <class 'list'>
    >>> len(params_generator.generated_params_list)
    0

    '''
    def __init__(self, parameters_structure, generated_params_list=None):
        self.parameters_structure = parameters_structure
        self.generated_params_list = generated_params_list

    # 生成参数组合列表
    def generate_params_list(self):
        parameters = http_params_generator.get_pairwise_list(self.get_params_num())
        params_usage_2d_list = []
        params_combo_list = []
        if len(parameters) > 1:
            for pairs in AllPairs(parameters):
                params_usage_2d_list.append(pairs)
            for params_usage_list in params_usage_2d_list:
                yield_params_usage_list = http_params_generator.yield_list(params_usage_list)
                raw_params_list = self.generate_params(parameters_usage_list=yield_params_usage_list)
                prepared_params_list = http_params_generator.get_value_dic(raw_params_list)
                params_combo_list.append(prepared_params_list)
        elif len(parameters) == 1:  # 当只有一个参数的时候第三方库ALLPAIRS不支持
            yield_params_usage_list_true = http_params_generator.yield_list([True])
            yield_params_usage_list_false = http_params_generator.yield_list([False])
            raw_params_list_true = self.generate_params(parameters_usage_list=yield_params_usage_list_true)
            prepared_params_list_true = http_params_generator.get_value_dic(raw_params_list_true)
            raw_params_list_false = self.generate_params(parameters_usage_list=yield_params_usage_list_false)
            prepared_params_list_false = http_params_generator.get_value_dic(raw_params_list_false)
            params_combo_list.append(prepared_params_list_true)
            params_combo_list.append(prepared_params_list_false)
        self.generated_params_list = params_combo_list

    # 生成参数
    def generate_params(self, parameters_usage_list, parameters_structure=None):
        if parameters_structure is None:
            parameters_structure = copy.deepcopy(self.parameters_structure)
        for key, attribute in parameters_structure.items():
            type_name = attribute['type']
            if type_name.lower() == 'object':
                self.generate_params(parameters_structure=attribute['value'],
                                     parameters_usage_list=parameters_usage_list)
                continue
            if type_name.lower() == 'array':
                for value in attribute['value']:
                    self.generate_params(parameters_structure=value,
                                         parameters_usage_list=parameters_usage_list)
                continue
            type_category = self.get_type_category(key)
            if 'range' in attribute and attribute['range']:
                generated_value = random.choice(attribute['range'])
            else:
                generated_value = self.get_parameter_random_value(type_name, type_category)

            if next(parameters_usage_list) or ('range' in attribute and attribute['range']):
                parameters_structure[key]['value'] = generated_value
            else:
                parameters_structure[key]['value'] = None
        return parameters_structure

    def get_params_num(self, parameters_structure=None, num=0):
        if parameters_structure is None:
            parameters_structure = copy.deepcopy(self.parameters_structure)
        for key, attribute in parameters_structure.items():
            type_name = attribute['type']
            if type_name.lower() == 'object':
                num += self.get_params_num(attribute['value'])
                continue
            if type_name.lower() == 'array':
                for value in attribute['value']:
                    num += self.get_params_num(value)
                continue
            else:
                num += 1
        return num

    # 比较两个字典部分是否相等
    @staticmethod
    def remove_duplicated_dict_in_list(dic_list):
        for dic in dic_list:
            pass

    @staticmethod
    def yield_list(input_list):
        for i in input_list:
            yield i

    @staticmethod
    def get_pairwise_list(params_num):
        parameters = []
        for i in range(params_num):
            parameters.append([True, False])
        return parameters

    # 返回仅提取输入字典值中的value的值作为当前键的值的字典
    @staticmethod
    def get_value_dic(dic):
        new_dic = dict()
        for key, attribute in dic.items():
            if attribute['type'].lower() == 'object':
                new_dic[key] = http_params_generator.get_value_dic(attribute['value'])
                continue
            if attribute['type'].lower() == 'array':
                new_dic[key] = []
                for value in attribute['value']:
                    new_dic[key].append(http_params_generator.get_value_dic(value))
                continue
            new_dic[key] = attribute['value']
        return new_dic

    # 根据键名生成数据类别
    @staticmethod
    def get_type_category(key):
        if key == 'phone':
            return 'chinese_mobile_phone'
        if key == 'name':
            return 'chinese_name'
        else:
            return 'default'

    # 根据数据类型以及数据类型的类别生成随机数据
    @staticmethod
    def get_parameter_random_value(type_name, type_category='default'):
        if type_name.lower() == 'boolean':
            return randoms.get_random_boolean()
        if type_name.lower() == 'number':
            return randoms.get_random_num(num_type=type_category)
        if type_name.lower() == 'string':
            return randoms.get_random_str(str_type=type_category)
        if type_name.lower() == 'date':
            return randoms.get_random_num(length=9)
        else:
            return None


if __name__ == '__main__':
    import doctest
    doctest.testmod()



import sys
sys.path.append("../..")
from testcase_automaker.interface.http_params_generator import http_params_generator
from testcase_automaker.Utils import httptools
from ptest.plogger import preporter
import requests
import copy
import random


class http_tester(object):
    '''

    >>> tester = http_tester(url='http://www.baidu.com', method='get',\
                            parameters_structure={'name': {'type': 'string', 'value': '', 'iscompulsory': True},\
                                                  'phone': {'type': 'number', 'value': '', 'iscompulsory': True}})
    >>> type(tester.pre_send_params_list)
    <class 'list'>
    >>> type(random.choice(tester.pre_send_params_list))
    <class 'dict'>
    >>> type(tester.url_list_for_method_get)
    <class 'list'>
    >>> tester.url in random.choice(tester.url_list_for_method_get)
    True

    >>> tester = http_tester(url='http://www.baidu.com', method='get',parameters_structure={})
    >>> len(tester.url_list_for_method_get)
    1
    >>> tester.url_list_for_method_get[0] == tester.url
    True

    '''

    def __init__(self, url, method, parameters_structure=None, session=None, headers=None):

        self.url = url
        self.method = method
        self.parameters_structure = parameters_structure
        self.session = session
        self.headers = headers

        if session is None:
            self.session = requests.Session()

        if parameters_structure is None:
            self.parameters_structure = dict()

        if isinstance(self.parameters_structure, dict):
            params_generator = http_params_generator(parameters_structure=self.parameters_structure)
            params_generator.generate_params_list()
            self.pre_send_params_list = params_generator.generated_params_list
        else:
            self.pre_send_params_list = None
            raise Exception('parameters_structure is not a dict！')

        # 若方法为get, 则构建请求url列表
        if self.pre_send_params_list is not None and self.method.lower() == 'get':
            url_list = []
            if self.pre_send_params_list:
                for pre_send_params in self.pre_send_params_list:
                    url = copy.deepcopy(self.url)
                    url += '?'
                    for key, value in pre_send_params.items():
                        if value is not None:
                            url += str(key) + '=' + str(value) + '&'
                    url = url[0:(len(url) - 1)]
                    url_list.append(url)
            else:
                url_list.append(self.url)
            self.url_list_for_method_get = url_list

    # 执行测试, 目前依附于ptest测试框架
    def exec_test(self, assertion=None):
        self.returned_data = []
        if self.method.lower() == 'get':
            for url in self.url_list_for_method_get:
                preporter.info('\n\n正在请求: ' + url + '\n\n')
                self.returned_data.append(
                    httptools.request(url=url, method=self.method, session=self.session, headers=self.headers,
                                      assertion=assertion))
        else:
            if len(self.pre_send_params_list) > 0:
                for pre_send_params in self.pre_send_params_list:
                    preporter.info('\n\n正在请求: ' + self.url + '\n\n' + '当前请求参数: ' + str(pre_send_params) + '\n\n')
                    self.returned_data.append(
                        httptools.request(url=self.url, method=self.method, json_data=pre_send_params,
                                          session=self.session, headers=self.headers, assertion=assertion))
            else:
                preporter.info('\n\n正在请求: ' + self.url + '\n\n')
                self.returned_data.append(httptools.request(url=self.url, method=self.method,
                                                            session=self.session, headers=self.headers, assertion=assertion))


if __name__ == '__main__':
    import doctest
    doctest.testmod()




from ptest.assertion import assert_equals, fail
from ptest.plogger import preporter
import json


def request(url, method, json_data=None, session=None, headers=None, assertion=None):
    data = ''
    status_code = ''
    req_json = ''
    try:
        text = session.request(url=url, method=method, json=json_data, headers=headers).text
        if len(text) <= 1000:
            preporter.info(text)
        req_json = json.loads(text)
        status_code = req_json["status"]
        data = req_json["data"]
        preporter.info('接口返回status为:' + str(status_code))
        if len(str(data)) >= 3000:
            preporter.info('接口返回data为(限制长度为3000):' + str(data)[0:3000])
        else:
            preporter.info('接口返回data为:' + str(data))
    except BaseException:
        if not assertion:
            fail('无法获取status或data！')
        else:
            key = assertion.keys()
            assert_equals(assertion[key], req_json[key])
    finally:
        if not assertion:
            if not status_code == 'ok':
                preporter.error('接口响应:' + str(req_json))
            else:
                assert_equals(status_code, 'ok')
            assert_equals(status_code, 'ok')
        if data is not None:
            return data
